@@ -30,6 +30,7 @@ from nomad import config, infrastructure
 from .optimade import optimade_app
 from .flask import app as flask_app
 from .v1.main import app as v1_app
+from .dcat.main import app as dcat_app
 
 
 class OasisAuthenticationMiddleware(BaseHTTPMiddleware):
@@ -42,9 +43,9 @@ class OasisAuthenticationMiddleware(BaseHTTPMiddleware):
             return Response(
                 status_code=status.HTTP_401_UNAUTHORIZED,
                 content='You have to authenticate to use this Oasis endpoint.')
-
         else:
-            user, _ = infrastructure.keycloak.auth(request.headers)
+            token = request.headers['Authorization'].split(' ')[1]
+            user, _ = infrastructure.keycloak.tokenauth(token)
             if user is None or user.email not in config.oasis.allowed_users:
                 return Response(
                     status_code=status.HTTP_401_UNAUTHORIZED,
@@ -60,12 +61,16 @@ if config.oasis.allowed_users is not None:
 
 app_base = config.services.api_base_path
 app.mount(f'{app_base}/api/v1', v1_app)
+app.mount(f'{app_base}/dcat', dcat_app)
 app.mount(f'{app_base}/optimade', optimade_app)
 app.mount(app_base, WSGIMiddleware(flask_app))
 
 
 @app.on_event('startup')
 async def startup_event():
+    from nomad.parsing.parsers import import_all_parsers
+    import_all_parsers()
+
     from nomad import infrastructure
     # each subprocess is supposed disconnect connect again: https://jira.mongodb.org/browse/PYTHON-2090
     try:
@@ -96,10 +101,9 @@ async def http_exception_handler(request, exc):
                 <h2>info</h2>
                 {'<br/>'.join(f'{key}: {value}' for key, value in config.meta.items())}
                 <h2>apis</h2>
-                <a href="{app_base}/api">NOMAD API v0</a><br/>
                 <a href="{app_base}/api/v1/extensions/docs">NOMAD API v1</a><br/>
                 <a href="{app_base}/optimade/v1/extensions/docs">Optimade API</a><br/>
-                <a href="{app_base}/dcat">DCAT API</a><br/>
+                <a href="{app_base}/dcat/extensions/docs">DCAT API</a><br/>
             </body>
         </html>
         ''')
@@ -109,10 +113,6 @@ async def http_exception_handler(request, exc):
         'info': {
             'app': config.meta,
             'apis': {
-                'v0': {
-                    'root': f'{app_base}/api',
-                    'dashboard': f'{app_base}/api',
-                },
                 'v1': {
                     'root': f'{app_base}/api/v1',
                     'dashboard': f'{app_base}/api/v1/extensions/docs',
@@ -124,7 +124,7 @@ async def http_exception_handler(request, exc):
                 },
                 'dcat': {
                     'root': f'{app_base}/dcat',
-                    'dashboard': f'{app_base}/dcat'
+                    'dashboard': f'{app_base}/dcat/extensions/docs'
                 }
             }
         }
