@@ -1,4 +1,4 @@
-#
+
 # Copyright The NOMAD Authors.
 #
 # This file is part of NOMAD. See https://nomad-lab.eu for further info.
@@ -20,7 +20,7 @@ import sys
 import os
 import click
 
-from nomad.cli.cli import cli
+from .cli import cli
 
 
 @cli.group(help='Commands related to the nomad source code.')
@@ -66,93 +66,104 @@ def gui_qa(skip_tests: bool):
 @dev.command(help='Generates a JSON with all metainfo.')
 def metainfo():
     import json
-    print(json.dumps(metainfo_undecorated(), indent=2))
+    export = _all_metainfo_packages()
+    metainfo_data = export.m_to_dict(with_meta=True)
+    print(json.dumps(metainfo_data, indent=2))
 
 
-def metainfo_undecorated():
+def _all_metainfo_packages():
     from nomad.metainfo import Package, Environment
-
-    # TODO the __init_metainfo__() should not be necessary and automatically performed
-    # Also load and initialize the datamodel definitions
-    import nomad.metainfo.metainfo
-    import nomad.datamodel.datamodel
-    import nomad.datamodel.dft
-    import nomad.datamodel.ems
-    import nomad.datamodel.optimade
-    import nomad.datamodel.encyclopedia
-    nomad.metainfo.metainfo.m_package.__init_metainfo__()
-    nomad.datamodel.datamodel.m_package.__init_metainfo__()
-    nomad.datamodel.dft.m_package.__init_metainfo__()  # pylint: disable=no-member
-    nomad.datamodel.ems.m_package.__init_metainfo__()  # pylint: disable=no-member
-    nomad.datamodel.optimade.m_package.__init_metainfo__()  # pylint: disable=no-member
-    nomad.datamodel.encyclopedia.m_package.__init_metainfo__()
+    from nomad.datamodel import EntryArchive
 
     # TODO similar to before, due to lazyloading, we need to explicily access parsers
     # to actually import all parsers and indirectly all metainfo packages
-    from nomad.parsing import parsers
-    parsers.parsers
+    from nomad.parsing.parsers import import_all_parsers
+    import_all_parsers()
+
+    # Create the ES mapping to populate ES annoations with search keys.
+    from nomad.search import entry_type
+    entry_type.create_mapping(EntryArchive.m_def)
+
+    # TODO we call __init_metainfo__() for all packages where this has been forgotten
+    # by the package author. Ideally this would not be necessary and we fix the
+    # actual package definitions.
+    for module_key in list(sys.modules):
+        pkg: Package = getattr(sys.modules[module_key], 'm_package', None)
+        if pkg is not None and isinstance(pkg, Package):
+            if (pkg.name not in Package.registry):
+                pkg.__init_metainfo__()
 
     export = Environment()
     for package in Package.registry.values():
         export.m_add_sub_section(Environment.packages, package)
-
-    return export.m_to_dict(with_meta=True)
+    return export
 
 
 @dev.command(help='Generates a JSON with all search quantities.')
 def search_quantities():
-    from nomad import search
-    # Due to this import, the parsing module will register all code_names based on parser
-    # implementations.
-    from nomad.parsing.parsers import parser_dict  # pylint: disable=unused-import
+    _all_metainfo_packages()
     import json
 
-    def to_dict(search_quantity):
-        result = {
-            'name': search_quantity.qualified_name,
-            'description': search_quantity.description,
-            'many': search_quantity.many,
-        }
-
-        if search_quantity.statistic_fixed_size is not None:
-            result['statistic_size'] = search_quantity.statistic_fixed_size
-        if search_quantity.statistic_values is not None:
-            result['statistic_values'] = search_quantity.statistic_values
+    # Currently only quantities with "entry_type" are included.
+    from nomad.metainfo.elasticsearch_extension import entry_type, Elasticsearch
+    from nomad.datamodel import EntryArchive
 
+    def to_dict(search_quantity, section=False):
+        if section:
+            keys = ["name", "description", "nested"]
+            metadict = search_quantity.sub_section.m_to_dict(with_meta=True)
+            metadict["name"] = search_quantity.m_to_dict(with_meta=True)["name"]
+            es_annotations = search_quantity.m_get_annotations(Elasticsearch, as_list=True)
+            nested = any([x.nested for x in es_annotations])
+            metadict["nested"] = nested
+        else:
+            keys = ["name", "description", "type", "unit"]
+            metadict = search_quantity.definition.m_to_dict(with_meta=True)
+        result = {}
+        for key in keys:
+            val = metadict.get(key)
+            if val is not None:
+                result[key] = val
         return result
 
-    export = {
-        search_quantity.qualified_name: to_dict(search_quantity)
-        for search_quantity in search.search_quantities.values()
-    }
+    export = {}
+
+    # Add quantities
+    for search_quantity in entry_type.quantities.values():
+        isSuggestion = search_quantity.annotation.suggestion
+        if not isSuggestion:
+            export[search_quantity.qualified_name] = to_dict(search_quantity)
+
+    # Add suggestion flag
+    for suggestion in entry_type.suggestions.keys():
+        export[suggestion]["suggestion"] = True
+
+    # Add section definitions
+    def get_sections(m_def, prefix=None):
+        for sub_section_def in m_def.all_sub_sections.values():
+            name = sub_section_def.name
+            full_name = f"{prefix}.{name}" if prefix else name
+            export[full_name] = to_dict(sub_section_def, True)
+            get_sections(sub_section_def.sub_section, full_name)
+    get_sections(EntryArchive.results.sub_section, "results")
+
     print(json.dumps(export, indent=2))
 
 
 @dev.command(help='Generates a JSON file that compiles all the parser metadata from each parser project.')
 def parser_metadata():
-    import inspect
-    import re
     import json
     import yaml
-
-    from nomad.parsing import Parser
-    from nomad.parsing.parsers import parser_dict
+    import os
+    import os.path
+    from glob import glob
 
     parsers_metadata = {}
-    for parser in parser_dict.values():
-        if isinstance(parser, Parser):
-            parser_class = parser.__class__
-        else:
-            continue
-        parser_code_file = inspect.getfile(parser_class)
-
-        path_match = re.match(r'(.*/dependencies/parsers/[^/]+)/.*', parser_code_file)
-        if path_match:
-            parser_metadata_file = os.path.join(path_match.group(1), 'metadata.yaml')
-            if os.path.exists(parser_metadata_file):
-                with open(parser_metadata_file) as f:
-                    parser_metadata = yaml.load(f, Loader=yaml.FullLoader)
-                parsers_metadata[parser.code_name] = parser_metadata
+    parsers_path = './dependencies/parsers'
+    for parser_metadata_file in sorted(glob(f'{parsers_path}/**/metadata.yaml', recursive=True)):
+        with open(parser_metadata_file) as f:
+            parser_metadata = yaml.load(f, Loader=yaml.FullLoader)
+        parsers_metadata[os.path.basename(os.path.dirname(parser_metadata_file))] = parser_metadata
 
     parsers_metadata = {
         key: parsers_metadata[key]
@@ -203,66 +214,117 @@ def update_parser_readmes(parser):
     generic_fn = './README.parsers.md'
     parser_path = './dependencies/parsers/'
 
-    for num, ddir in enumerate(sorted(glob(parser_path + '*/')), start=1):
-        if parser is not None and parser != ddir.split(os.sep)[-2]:
-            print(f'Skip {ddir}')
-            continue
+    # Open general template
+    with open(generic_fn, 'r') as generic:  # read only
+        generic_contents = generic.read()
 
-        _, parser_dirname = os.path.split(ddir)
-        print('{} Working on {}' .format(num, parser_dirname))
+    # Replace the comment at the top of the gereral template
+    generic_contents = re.sub(
+        rf'\*\*\*Note:\*\* This is a general README file for NOMAD parsers, '
+        rf'consult the README of specific parser projects for more detailed '
+        rf'information!\*\n\n', '', generic_contents)
 
-        # Open general template
-        with open(generic_fn, 'r') as generic:  # read only
-            body = generic.read()
+    def open_metadata(path):
+        # read local yaml metadata file
+        with open(path, 'r') as metadata_f:
+            try:
+                metadata = yaml.load(metadata_f, Loader=yaml.FullLoader)
+            except Exception as e:
+                print(f'Error reading metadata.yaml: {e}')
+                metadata = None
+        return metadata
 
-        # Open local YAML metadata file
-        local_mdata = ddir + 'metadata.yaml'
-        with open(local_mdata, 'r') as mdata_f:
-            mdata = yaml.load(mdata_f, Loader=yaml.FullLoader)
-            if mdata.get('codeName', '').strip() == '':
-                mdata['codeName'] = os.path.basename(os.path.dirname(ddir))
-            if 'preamble' not in mdata:
-                mdata['preamble'] = ''
+    def replace(metadata, contents, path):
+        # replace placelder in contents with metadata values
+        for key in metadata.keys():
+            replace = metadata.get(key)
+            if replace is None:
+                continue
+            print(f'\tReplacing {key} with {replace}')
+            contents = re.sub(rf'\${key}\$', replace, contents)
+
+        # save file
+        with open(path, 'w') as f:
+            f.write(contents.strip())
+            f.write('\n')
+
+    for local_readme in sorted(glob(f'{parser_path}/*/{local_fn}')):
+        parser_dir = os.path.dirname(local_readme)
+        project_name = os.path.basename(parser_dir)
+        print(f'Working on {parser_dir}')
+
+        contents = generic_contents
+        # if metadata file under the parser directory exists, it is a single parser project
+        single = os.path.isfile(os.path.join(parser_dir, 'metadata.yaml'))
+
+        if single:
+            metadata = open_metadata(os.path.join(parser_dir, 'metadata.yaml'))
+            # git path is given by nomad-parser-codename
+            metadata['gitPath'] = f'nomad-parser-{project_name}'
+            parser_header, parser_specs = '', ''
+        else:
+            # replace header for the single parser with that for a group of parsers
+            parser_header_re = r'(\nThis is a NOMAD parser[\s\S]+?Archive format\.\n)'
+            parser_header = re.search(parser_header_re, contents).group(1)
+            group_header = 'This is a collection of the NOMAD parsers for the following $codeName$ codes:\n\n$parserList$'
+            contents = re.sub(parser_header_re, group_header, contents)
+            # remove individual parser specs
+            parser_specs_re = r'(For \$codeLabel\$ please provide[\s\S]+?\$tableOfFiles\$)\n\n'
+            parser_specs = re.search(parser_specs_re, contents).group(1)
+            contents = re.sub(parser_specs_re, '', contents)
+            metadata = dict(
+                gitPath=f'{project_name}-parsers',
+                parserGitUrl=f'https://github.com/nomad-coe/{project_name}-parsers.git',
+                parserSpecific='')
+        if metadata.get('codeName', '').strip() == '':
+            metadata['codeName'] = project_name
+        if 'preamble' not in metadata:
+            metadata['preamble'] = ''
+
+        # if this is a group of parser, find all individdual parsers and write the
+        # parser specs
+        parser_list = ''
+        for index, local_metadata in enumerate(sorted(glob(f'{parser_dir}/*/*/metadata.yaml'))):
+            metadata_parser = open_metadata(local_metadata)
+            # contents is simply the parser header and specs
+            contents_parser = f'{parser_header}\n{parser_specs}'
+            replace(metadata_parser, contents_parser, os.path.join(os.path.dirname(local_metadata), local_fn))
+            # add the codename to the list of parsers for the group header
+            codelabel = metadata_parser.get('codeLabel', os.path.basename(os.path.dirname(local_metadata)))
+            codeurl = metadata_parser.get('codeUrl', '')
+            parser_list = rf'{parser_list}{index + 1}. [{codelabel}]({codeurl})\n'
+        metadata['parserList'] = parser_list.strip()
 
         # Find & Replace Parser`s metadata on its README file
-        local_readme = ddir + local_fn
-        with open(local_readme, 'w') as local:
-            for key in mdata.keys():
-                ignore = ['codeLabelStyle', 'parserDirName']
-                if key in ignore:
-                    continue
+        replace(metadata, contents, local_readme)
 
-                find = r'\$' + key + r'\$'
-                replace = mdata[key]
-                print(f'\tReplacing {key} with {replace}')
 
-                if key == 'parserSpecific':
-                    if mdata[key] != '':
-                        replace = r'## Parser Specific\n' + replace
+@dev.command(help='Adds a few pieces of data to NOMAD.')
+@click.option('--username', '-u', type=str, help='The main author username.')
+def example_data(username: str):
+    from nomad import infrastructure, utils
+    from tests.utils import ExampleData
 
-                body = re.sub(find, replace, body)
+    infrastructure.setup()
 
-            # Extra: to replace the codeName (there's no YAML key for this)
-            key = 'codeName'
-            print('\tReplacing', key)
-            find = r'\$' + key + r'\$'
-            replace = parser_dirname
-            body = re.sub(find, replace, body)
+    main_author = infrastructure.keycloak.get_user(username=username)
+    if main_author is None:
+        print(f'The user {username} does not exist.')
+        sys.exit(1)
 
-            # Extra: to replace the comment at the top of the gereral template
-            find = (
-                r'\*\*\*Note:\*\* This is a general README file for NOMAD parsers, '
-                r'consult the README of specific parser projects for more detailed '
-                r'information!\*\n\n')
-            replace = ''
-            print('\tReplacing the top comment')
-            # print('\t', find, ' -> ', replace)
-            body = re.sub(find, replace, body)
+    data = ExampleData(main_author=main_author)
 
-            # save file
-            local.seek(0)  # go to the top
-            local.write(body.strip())
-            local.truncate()
+    # one upload with two entries published with embargo, one shared
+    upload_id = utils.create_uuid()
+    data.create_upload(upload_id=upload_id, published=True, embargo_length=0)
+    data.create_entry(
+        entry_id=utils.create_uuid(),
+        upload_id=upload_id,
+        mainfile='test_content/test_embargo_entry/mainfile.json')
+
+    data.save(with_files=True, with_es=True, with_mongo=True)
+
+    return data
 
 
 @dev.command(help='Creates a Javascript source file containing the required unit conversion factors.')
@@ -283,7 +345,7 @@ def units(ctx):
         "atomic_unit_of_time": {
             "dimension": "time",
             "label": "Atomic unit of time",
-            "abbreviation": "atomic_unit_of_time",
+            "abbreviation": "a_u_time",
         },
         # Length
         "meter": {
@@ -326,7 +388,7 @@ def units(ctx):
         "atomic_unit_of_current": {
             "dimension": "current",
             "label": "Atomic unit of current",
-            "abbreviation": "atomic_unit_of_current",
+            "abbreviation": "a_u_current",
         },
         # Substance
         "mole": {
@@ -359,7 +421,7 @@ def units(ctx):
         "atomic_unit_of_temperature": {
             "dimension": "temperature",
             "label": "Atomic unit of temperature",
-            "abbreviation": "atomic_unit_of_temperature",
+            "abbreviation": "a_u_temperature",
         },
         # Force
         "newton": {
@@ -370,7 +432,7 @@ def units(ctx):
         "atomic_unit_of_force": {
             "dimension": "force",
             "label": "Atomic unit of force",
-            "abbreviation": "atomic_unit_of_force",
+            "abbreviation": "a_u_force",
         },
         # Pressure
         "pascal": {
@@ -378,6 +440,16 @@ def units(ctx):
             "label": "Pascal",
             "abbreviation": "Pa"
         },
+        "gigapascal": {
+            "dimension": "pressure",
+            "label": "Gigapascal",
+            "abbreviation": "GPa"
+        },
+        "atomic_unit_of_pressure": {
+            "dimension": "pressure",
+            "label": "Atomic unit of pressure",
+            "abbreviation": "a_u_pressure"
+        },
         # Energy
         "joule": {
             "dimension": "energy",
@@ -441,12 +513,29 @@ def units(ctx):
             "label": "Weber",
             "abbreviation": "Wb",
         },
+        # Magnetic dipole
+        "bohr_magneton": {
+            "dimension": "magnetic_dipole",
+            "label": "Bohr magneton",
+            "abbreviation": "Bm",
+        },
         # Inductance
         "henry": {
             "dimension": "inductance",
             "label": "Henry",
             "abbreviation": "H",
         },
+        # angle
+        "radian": {
+            "dimension": "angle",
+            "label": "Radian",
+            "abbreviation": "rad",
+        },
+        "degree": {
+            "dimension": "angle",
+            "label": "Degree",
+            "abbreviation": "°",
+        },
         # dimensionless
         "dimensionless": {
             "dimension": "dimensionless",
@@ -521,6 +610,8 @@ def units(ctx):
         "pressure": {
             "units": [
                 "pascal",
+                "gigapascal",
+                "atomic_unit_of_pressure",
             ],
             "multipliers": {},
         },
@@ -575,6 +666,12 @@ def units(ctx):
             ],
             "multipliers": {},
         },
+        "magnetic_dipole": {
+            "units": [
+                "bohr_magneton",
+            ],
+            "multipliers": {},
+        },
         "inductance": {
             "dimension": "inductance",
             "units": [
@@ -582,6 +679,14 @@ def units(ctx):
             ],
             "multipliers": {},
         },
+        "angle": {
+            "dimension": "angle",
+            "units": [
+                "radian",
+                "degree",
+            ],
+            "multipliers": {},
+        },
         "dimensionless": {
             "dimension": "dimensionless",
             "units": [
@@ -611,6 +716,7 @@ def units(ctx):
                 "frequency": "hertz",
                 "electric_potential": "volt",
                 "charge": "coulomb",
+                "angle": "radian",
             },
         },
         "AU": {
@@ -624,6 +730,8 @@ def units(ctx):
                 "temperature": "atomic_unit_of_temperature",
                 "force": "atomic_unit_of_force",
                 "energy": "hartree",
+                "pressure": "atomic_unit_of_pressure",
+                "angle": "radian",
             }
         }
     }
@@ -677,11 +785,11 @@ def units(ctx):
             assert dimension == info["dimension"]
 
     # Go through the metainfo and check that all units are defined
-    all_metainfo = metainfo_undecorated()
+    all_metainfo = _all_metainfo_packages()
     units = set()
-    packages = all_metainfo["packages"]
+    packages = all_metainfo.m_to_dict(with_meta=True)['packages']
     for package in packages:
-        sections = package["section_definitions"]
+        sections = package.get("section_definitions", [])
         for section in sections:
             quantities = section.get("quantities", [])
             for quantity in quantities:
@@ -698,6 +806,9 @@ def units(ctx):
                         if not is_operator and not is_number:
                             units.add(part)
     for unit in units:
+        if unit == 'byte':
+            continue
+
         assert unit in unit_map, "The unit '{}' is not defined in the unit definitions.".format(unit)
 
     # Print unit conversion table and unit systems as a Javascript source file
