@@ -22,9 +22,10 @@ import re
 import os
 import os.path
 from functools import lru_cache
+import importlib
 
-from nomad import config
-from nomad.datamodel import EntryArchive, UserProvidableMetadata, EntryMetadata
+from nomad import config, utils
+from nomad.datamodel import EntryArchive, EntryMetadata
 
 
 class Parser(metaclass=ABCMeta):
@@ -33,6 +34,11 @@ class Parser(metaclass=ABCMeta):
     and extracted files. Further, allows to run the parser on those 'main files'.
     '''
     name = "parsers/parser"
+    level = 0
+    '''
+    Level 0 parsers are run first, then level 1, and so on. Normally the value should be 0,
+    use higher values only when a parser depends on other parsers.
+    '''
 
     def __init__(self):
         self.domain = 'dft'
@@ -61,7 +67,19 @@ class Parser(metaclass=ABCMeta):
         Args:
             mainfile: A path to a mainfile that this parser can parse.
             archive: An instance of the section :class:`EntryArchive`. It might contain
-                a ``section_metadata`` with information about the entry.
+                a section ``metadata`` with information about the entry.
+            logger: A optional logger
+        '''
+        pass
+
+    def after_normalization(self, archive: EntryArchive, logger=None) -> None:
+        '''
+        This is called after the archive produced by `parsed` has been normalized. This
+        allows to apply additional code-specific processing steps based on the normalized data.
+
+        Args:
+            archive: An instance of the section :class:`EntryArchive`. It might contain
+                a section ``metadata`` with information about the entry.
             logger: A optional logger
         '''
         pass
@@ -124,6 +142,7 @@ class MatchingParser(Parser):
             self, name: str, code_name: str, code_homepage: str = None,
             mainfile_contents_re: str = None,
             mainfile_binary_header: bytes = None,
+            mainfile_binary_header_re: bytes = None,
             mainfile_mime_re: str = r'text/.*',
             mainfile_name_re: str = r'.*',
             mainfile_alternative: bool = False,
@@ -144,6 +163,10 @@ class MatchingParser(Parser):
             self._mainfile_contents_re = re.compile(mainfile_contents_re)
         else:
             self._mainfile_contents_re = None
+        if mainfile_binary_header_re is not None:
+            self._mainfile_binary_header_re = re.compile(mainfile_binary_header_re)
+        else:
+            self._mainfile_binary_header_re = None
         self._supported_compressions = supported_compressions
 
         self._ls = lru_cache(maxsize=16)(lambda directory: os.listdir(directory))
@@ -156,6 +179,13 @@ class MatchingParser(Parser):
             if self._mainfile_binary_header not in buffer:
                 return False
 
+        if self._mainfile_binary_header_re is not None:
+            if buffer is not None:
+                if self._mainfile_binary_header_re.search(buffer) is None:
+                    return False
+            else:
+                return False
+
         if self._mainfile_contents_re is not None:
             if decoded_buffer is not None:
                 if self._mainfile_contents_re.search(decoded_buffer) is None:
@@ -191,8 +221,42 @@ class MatchingParser(Parser):
         return self.name
 
 
-# For backward compatibility
-FairdiParser = MatchingParser
+class MatchingParserInterface(MatchingParser):
+    '''
+    An interface to the NOMAD parsers.
+
+    Arguments:
+        parser_class_name: concatenation of module path and parser class name
+    '''
+    def __init__(self, parser_class_name: str, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._parser_class_name = parser_class_name
+        self._mainfile_parser = None
+
+    @property
+    def mainfile_parser(self):
+        if self._mainfile_parser is None:
+            try:
+                Parser = self.import_parser_class()
+                self._mainfile_parser = Parser()
+            except Exception as e:
+                logger = utils.get_logger(__name__)
+                logger.error('cannot instantiate parser.', exc_info=e)
+                raise e
+        return self._mainfile_parser
+
+    def parse(self, mainfile: str, archive: EntryArchive, logger=None):
+        self.mainfile_parser.parse(mainfile, archive, logger)
+
+    def import_parser_class(self):
+        try:
+            module_path, parser_class = self._parser_class_name.rsplit('.', 1)
+            module = importlib.import_module(module_path)
+            return getattr(module, parser_class)
+        except Exception as e:
+            logger = utils.get_logger(__name__)
+            logger.error('cannot import parser', exc_info=e)
+            raise e
 
 
 class ArchiveParser(MatchingParser):
@@ -201,25 +265,24 @@ class ArchiveParser(MatchingParser):
             name='parsers/archive',
             code_name=config.services.unavailable_value,
             domain=None,
-            mainfile_mime_re=r'application/json',
-            mainfile_name_re=r'.*(archive|metainfo)\.json',
-            mainfile_contents_re=r'section_metadata|results')
+            mainfile_mime_re='.*',
+            mainfile_name_re=r'.*(archive|metainfo)\.(json|yaml|yml)$')
 
     def parse(self, mainfile: str, archive: EntryArchive, logger=None):
-        import json
-        with open(mainfile, 'rt') as f:
-            archive_data = json.load(f)
+        if mainfile.endswith('.json'):
+            import json
+            with open(mainfile, 'rt') as f:
+                archive_data = json.load(f)
+        else:
+            import yaml
+            with open(mainfile, 'rt') as f:
+                archive_data = yaml.load(f, Loader=getattr(yaml, 'FullLoader'))
 
-        metadata_data = archive_data.get(EntryArchive.section_metadata.name, None)
+        metadata_data = archive_data.get(EntryArchive.metadata.name, None)
 
         if metadata_data is not None:
-            metadata = archive.section_metadata
-            for key, value in metadata_data.items():
-                if UserProvidableMetadata.m_def not in getattr(EntryMetadata, key).categories:
-                    continue
-
-                metadata.m_update_from_dict({key: value})
-            del(archive_data[EntryArchive.section_metadata.name])
+            # Setting metadata in this way is not supported (any more)
+            del(archive_data[EntryArchive.metadata.name])
 
         archive.m_update_from_dict(archive_data)
 
